using System;
using System.Web.Services.Protocols;
using System.Reflection;
using System.IO;

namespace Interval.Examples.SoapExtensions
{
	/// <summary>
	/// SOAP extenze pro sledovn komunikace mezi klientem a serverem
	/// </summary>
	public class SpySoapExtension : SoapExtension
	{
		
		#region private fields
		private string m_fileName;
		private Stream m_oldStream;
		private Stream m_newStream;
		#endregion private fields
		/// <summary>
		/// Standardn konstruktor
		/// </summary>
		public SpySoapExtension()
		{
			m_oldStream = null;
			m_newStream = null;
			m_fileName = "";
			
		}
		/// <summary>
		/// Pepsn metody GetInitializer
		/// </summary>
		/// <param name="methodInfo">Informace o metod, na kterou je extenze aplikovna</param>
		/// <param name="attribute">Instance tdy <see cref="SpySoapExtensionAttribute"/>, ve kter je uloen nzev souboru, do kterho maj bt ukldny SOAP zprvy</param>
		/// <returns>Nzev souboru, do kterho budou uloeny SOAP zprvy</returns>
		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute)
		{

			//Petypovn na SpyExtensionAttribute
			SpyExtensionAttribute spyAttribute = attribute as SpyExtensionAttribute;
			
			//Vrcen nzvu souboru, do kterho budou uloeny SOAP zprvy
			return spyAttribute.FileName;

		}
		/// <summary>
		/// Pepsn metody GetInitializer, kter je volna, pokud je SOAP extenze zaregistrovna ve web.config
		/// </summary>
		/// <param name="serviceType">Typ WWW sluby</param>
		/// <returns>Nzev souboru, do kterho budou uloeny SOAP zprvy</returns>
		public override object GetInitializer (Type serviceType)
		{
			//Vrcen nzvu souboru, do kterho budou uloeny SOAP zprvy
			return (serviceType.Assembly.FullName + ".spy");
		}

		/// <summary>
		/// Inicializace, kter probh pi kadm voln WWW metody
		/// </summary>
		/// <param name="initializer">objekt vrcen z metody <see cref="GetInitializer"/></param>
		public override void Initialize (object initializer)
		{
			//Uloen nzvu souboru do privtn promnn
			m_fileName = (string) initializer;
		}

		/// <summary>
		/// Metoda, ve kter uschovme referenci na pedan stream a vrtme nov stream
		/// </summary>
		/// <param name="stream">originln <see cref="Stream"/></param>
		/// <returns>Nov stream, do kterho mohou bt provdny zmny</returns>
		public override Stream ChainStream(Stream stream)
		{
			m_oldStream = stream;
			m_newStream = new MemoryStream();
			return m_newStream;

		}

		/// <summary>
		/// Zpracovn zprv podle fze poadavku+
		/// </summary>
		/// <param name="message">SOAP zprva</param>
		public override void ProcessMessage (SoapMessage message)
		{
			switch (message.Stage)
			{
				//Ulome SOAP zprvu, kter pila od klienta
				case SoapMessageStage.BeforeDeserialize:
				{
					WriteClientRequest();
					break;

				}
				//V tto fzi nepotenujeme nic zpracovvat
				case SoapMessageStage.AfterDeserialize:
				{
					break;
				}
				
				//V tto fzi nepotebujeme nic zpracovvat
				case SoapMessageStage.BeforeSerialize:
				{
					
					break;
				}
				
				//Ulome odpov ze serveru
				case SoapMessageStage.AfterSerialize:
				{
					WriteServerResponse();
					break;
				}


			}
		}

		/// <summary>
		/// Zapsn SOAP zprvy od klienta do souboru
		/// </summary>
		private void WriteClientRequest()
		{
			
			
			//Reset pozice
			m_oldStream.Position = 0;
			m_newStream.Position = 0;
			
			//Pekoprujeme obsah plvodnho streamu do novho streamu
			CopyStream(m_oldStream, m_newStream, false);
			
			//Pesun na zatek streamu
			m_newStream.Position = 0;
			
			//Oteven souboru, do kterho bude pidna SOAP zprva
			FileStream fs = new FileStream(m_fileName, FileMode.Append);
			
			//Zkoprovn streamu do souboru
			CopyStream(m_newStream, fs, true);
			//Uzaven souboru
			fs.Close();
			//Opt reset pozice, aby ze streamu mohl st ASP.NET runtime
			m_newStream.Position = 0;
			
			
		}
		
		private void WriteServerResponse()
		{
			
			//Pesun na zatek streamu
			m_newStream.Position = 0;
			
			//Oteven souboru, do kterho bude pidna SOAP zprva
			FileStream fs = new FileStream(m_fileName,FileMode.Append);
			
			//Zkoprovn streamu do souboru
			CopyStream(m_newStream, fs, true);
			
			//Uzaven souboru
			fs.Close();
			//Reset pozice
			m_newStream.Position = 0;
			
			//ASP.NET runtime te z pvodnho streamu
			CopyStream(m_newStream, m_oldStream, false);	
		}
		
		//Koprovn stream
		private void CopyStream (Stream from, Stream to, bool writeSeparator)
		{
			TextReader sr = new StreamReader(from);
			TextWriter sw = new StreamWriter (to);
			
			if (writeSeparator)
			{
				sw.WriteLine();
				sw.WriteLine(new String('-', 60));

			}
			sw.Write(sr.ReadToEnd());
			sw.Flush();
			
		}
			
	}
}
